<?php

namespace App\Security;

use App\Entity\User;
use App\Form\SecuritySignInForm;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Encoder\UserPasswordEncoderInterface;
use Symfony\Component\Security\Core\Exception\CustomUserMessageAuthenticationException;
use Symfony\Component\Security\Core\Security;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Security\Core\User\UserProviderInterface;
use Symfony\Component\Security\Guard\Authenticator\AbstractFormLoginAuthenticator;
use Symfony\Component\Security\Guard\PasswordAuthenticatedInterface;
use Symfony\Component\Security\Http\Util\TargetPathTrait;

class LoginFormAuthenticator extends AbstractFormLoginAuthenticator implements PasswordAuthenticatedInterface {
    use TargetPathTrait;

    private EntityManagerInterface       $entityManager;
    private UrlGeneratorInterface        $urlGenerator;
    private UserPasswordEncoderInterface $passwordEncoder;
    private FormFactoryInterface         $formFactory;

    public function __construct (
        EntityManagerInterface $entityManager,
        UrlGeneratorInterface $urlGenerator,
        UserPasswordEncoderInterface $passwordEncoder,
        FormFactoryInterface $formFactory
    ) {
        $this->entityManager = $entityManager;
        $this->urlGenerator = $urlGenerator;
        $this->passwordEncoder = $passwordEncoder;
        $this->formFactory = $formFactory;
    }

    public function supports (Request $request) {
        return 'app_security_sign_in' === $request->attributes->get('_route') && $request->isMethod('POST');
    }

    public function getCredentials (Request $request) {
        if (!$request->isMethod('POST') || $request->attributes->get('_route') != 'app_security_sign_in') {
            return null;
        }

        $form = $this->formFactory->create(SecuritySignInForm::class);
        $form->handleRequest($request);
        if (!$form->isSubmitted() || !$form->isValid()) {
            return null;
        }

        $credentials = $form->getData();
        $request->getSession()->set(
            Security::LAST_USERNAME,
            $credentials['email']
        );

        return $credentials;
    }

    public function getUser ($credentials, UserProviderInterface $userProvider) {
        $user = $this->entityManager->getRepository(User::class)->findOneBy(['email' => $credentials['email']]);
        if (!$user) {
            throw new CustomUserMessageAuthenticationException('Email could not be found.');
        }

        return $user;
    }

    public function checkCredentials ($credentials, UserInterface $user) {
        return $this->passwordEncoder->isPasswordValid($user, $credentials['password']);
    }

    /**
     * Used to upgrade (rehash) the user's password automatically over time.
     *
     * @param string[] $credentials Old informations
     *
     * @return string|null
     */
    public function getPassword ($credentials): ?string {
        return $credentials['password'];
    }

    public function onAuthenticationSuccess (Request $request, TokenInterface $token, $providerKey) {
        if ($targetPath = $this->getTargetPath($request->getSession(), $providerKey)) {
            return new RedirectResponse($targetPath);
        }

        return new RedirectResponse($this->urlGenerator->generate('app_site_index'));
    }

    protected function getLoginUrl () {
        return $this->urlGenerator->generate('app_security_sign_in');
    }
}
